/*
 * Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package com.baidubce.services.blb.model;

import java.util.List;

import com.baidubce.auth.BceCredentials;
import com.baidubce.model.AbstractBceRequest;
import com.fasterxml.jackson.annotation.JsonIgnore;

/**
 * The request for modify backendServer attributes.
 */
public class ModifyBSAttributesRequest extends AbstractBceRequest {

    /**
     * An ASCII string whose length is less than 64.
     * <p/>
     * The request will be idempotent if clientToken is provided.
     * If the clientToken is not specified by the user, a random String generated by default algorithm will be used.
     * See more detail at
     * <a href = "https://bce.baidu.com/doc/BCC/API.html#.E5.B9.82.E7.AD.89.E6.80.A7">
     * BCE API doc</a>
     */
    @JsonIgnore
    private String clientToken;

    /**
     * the id of the blb.
     */
    private String blbId;

    /**
     * the backendServers after modifying.
     */
    private List<BackendServer> backendServerList;

    public ModifyBSAttributesRequest withClientToken(String clientToken) {
        this.setClientToken(clientToken);
        return this;
    }

    public ModifyBSAttributesRequest withBlbId(String blbId) {
        this.setBlbId(blbId);
        return this;
    }

    public ModifyBSAttributesRequest withBackendServerList(List<BackendServer> backendServerList) {
        this.setBackendServerList(backendServerList);
        return this;
    }

    public ModifyBSAttributesRequest(String blbId, List<BackendServer> backendServerList) {
        this.blbId = blbId;
        this.backendServerList = backendServerList;
    }

    public String getClientToken() {
        return clientToken;
    }

    public void setClientToken(String clientToken) {
        this.clientToken = clientToken;
    }

    public String getBlbId() {
        return blbId;
    }

    public void setBlbId(String blbId) {
        this.blbId = blbId;
    }

    public List<BackendServer> getBackendServerList() {
        return backendServerList;
    }

    public void setBackendServerList(List<BackendServer> backendServerList) {
        this.backendServerList = backendServerList;
    }

    @Override
    public ModifyBSAttributesRequest withRequestCredentials(BceCredentials credentials) {
        this.setRequestCredentials(credentials);
        return this;
    }
}
